// pages/api/socket.ts

import type { Server as HTTPServer } from "http";
import type { Socket as NetSocket } from "net";
import type { NextApiRequest, NextApiResponse } from "next";
import type { Server as IOServer } from "socket.io";
import { Server } from "socket.io";
import http from "http";
import getConfig from "next/config";

export const config = {
  api: {
    bodyParser: false,
  },
};

const PORT = 5001;

interface SocketServer extends HTTPServer {
  io?: IOServer | undefined;
}

interface SocketWithIO extends NetSocket {
  server: SocketServer;
}

interface NextApiResponseWithSocket extends NextApiResponse {
  socket: SocketWithIO;
}
function getRoomId(data: any) {
  return data.organization_id + "-" + data.branch + "-" + data.user_id;
}

export default function SocketHandler(
  _req: NextApiRequest,
  res: NextApiResponseWithSocket
) {
  if (res.socket.server.io) {
    res.status(200).json({
      success: true,
      message: "Socket is already running",
      socket: `:${PORT + 1}`,
    });
    return;
  }

  console.log("Starting Socket.IO server' on port:", PORT + 1);

  const io = new Server({
    path: "/api/socket",
    addTrailingSlash: false,
    cors: { origin: "*" },
  }).listen(PORT + 1);

  io.on("connect", (socket) => {
    console.log("socket connect", socket.id);

    socket.emit("message", { message: "Hello from server!" });

    socket.on("JOIN_BRANCH", (data) => {
      try {
        var roomId =
          data.organization_id + "-" + data.branch + "-" + data.user_id;
        socket.join(roomId);
        socket.emit("message", { message: "Jonied to " + roomId });
      } catch (error) {
        socket.emit("message", {
          message: "Failed to Join - Bad Data - " + data,
        });
      }
    });

    socket.on("ORDER_PLACED", (data) => {
      socket.broadcast.to(getRoomId(data)).emit("ORDER_PLACED", data);
    });

    socket.on("ALERT_NOTIFICATION", (data) => {
      socket.broadcast.to(getRoomId(data)).emit("ALERT_NOTIFICATION", data);
    });

    socket.on("TABLE_CHANGE", (data) => {
      socket.broadcast.to(getRoomId(data)).emit("TABLE_CHANGE", data);
    });

    socket.on("TRACK_ORDER", (data) => {
      socket.broadcast.to(getRoomId(data)).emit("TRACK_ORDER", data);
    });

    socket.on("DELIVERY_ORDER", (data) => {
      socket.broadcast.to(getRoomId(data)).emit("DELIVERY_ORDER", data);
    });
    socket.on("DELIVERY_SUMMARY", (data) => {
      socket.broadcast.to(getRoomId(data)).emit("DELIVERY_SUMMARY", data);
    });

    socket.on("DINE_ORDERS", (data) => {
      socket.broadcast.to(getRoomId(data)).emit("DINE_ORDERS", data);
    });

    socket.on("message", (msg: any) => {
      socket.broadcast.emit("message", msg);
    });

    socket.on("disconnect", async () => {
      console.log("socket disconnect");
    });
  });

  res.socket.server.io = io;
  getConfig().serverRuntimeConfig.socketIo = io;

  res.status(201).json({
    success: true,
    message: "Socket is started",
    socket: `:${PORT + 1}`,
  });
}
